/* This file is part of mailfrom filter.
   Copyright (C) 2005, Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301 USA */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <mailutils/mailutils.h>
#include "mailfrom.h"

#ifdef USE_DBM

void
db_list_item(char *dbname, char *email, db_item_printer_t fun)
{	
	DBM_FILE db;
	DBM_DATUM key, contents;
	
	if (mu_dbm_open(dbname, &db, MU_STREAM_READ, 0600)) {
		mu_error("mu_dbm_open(%s) failed: %s", dbname,
			 mu_strerror(errno));
		return;
	}
	memset (&contents, 0, sizeof contents);
	memset(&key, 0, sizeof key);
	MU_DATUM_PTR(key) = email;
	MU_DATUM_SIZE(key) = strlen (email);
	if (mu_dbm_fetch(db, key, &contents) == 0) 
		fun(MU_DATUM_PTR(key),
		    MU_DATUM_SIZE(key),
		    MU_DATUM_PTR(contents));
	mu_dbm_close(db);
}

void
db_list(char *dbname, db_item_printer_t fun)
{
	DBM_FILE db;
	DBM_DATUM key, contents;
	
	if (mu_dbm_open(dbname, &db, MU_STREAM_READ, 0600)) {
		mu_error("mu_dbm_open(%s) failed: %s", dbname,
			 mu_strerror(errno));
		return;
	}
	for (key = mu_dbm_firstkey (db); MU_DATUM_PTR(key);
	     key = mu_dbm_nextkey (db, key)) {
		memset (&contents, 0, sizeof contents);
		mu_dbm_fetch (db, key, &contents);
		fun(MU_DATUM_PTR(key),
		    MU_DATUM_SIZE(key),
		    MU_DATUM_PTR(contents));
	}
	mu_dbm_close(db);
}

void
db_expire(char *dbname, db_expire_t fun)
{
	DBM_FILE db;
	DBM_DATUM key, contents;
	
	if (mu_dbm_open(dbname, &db, MU_STREAM_RDWR, 0600)) {
		mu_error("mu_dbm_open(%s) failed: %s", dbname,
			 mu_strerror(errno));
		return;
	}
	for (key = mu_dbm_firstkey (db); MU_DATUM_PTR(key);
	     key = mu_dbm_nextkey (db, key)) {
		memset (&contents, 0, sizeof contents);
		mu_dbm_fetch (db, key, &contents);
		if (fun(MU_DATUM_PTR(contents)))
			if (mu_dbm_delete (db, key)) {
				size_t size = MU_DATUM_SIZE(key);
				mu_error ("Cannot remove record for %*.*s",
					  size, size, MU_DATUM_PTR(key));
			}
	}
	mu_dbm_close(db);
}

void
db_delete(char *dbname, char *id)
{
	DBM_FILE db;
	DBM_DATUM key;
	
	if (mu_dbm_open(dbname, &db, MU_STREAM_RDWR, 0600)) {
		mu_error("mu_dbm_open(%s) failed: %s", dbname,
			 mu_strerror(errno));
		return;
	}
	memset(&key, 0, sizeof key);
	MU_DATUM_PTR(key) = id;
	MU_DATUM_SIZE(key) = strlen (id);
	if (mu_dbm_delete(db, key))
		mu_error("Cannot remove record for %s", id);
	mu_dbm_close(db);
}	

#endif
